#! /usr/bin/env python
from version import __appname__, __version__  # NOQA
import datetime
import os
import sys

from scoundrels import barf, cfg, logger
from scrib import process


if sys.version_info[0] >= 3:
	xrange = range
	unicode = str
	basestring = str

# Gross workaround.
botname = 'scrib'
try:
	botname = sys.argv[1]
except:
	pass


class Scrib(object):
	"""
	Set us up the scrib.
	"""
	def __init__(self, interface, args):
		"""
		Here we'll load settings and set up modules.
		"""
		try:
			self.name = args['name']
		except:
			self.name = 'scrib'

		self.cfgdir = '/.%s' % __appname__

		# Load scrib settings
		self.settings = self.setcfg('%s/%s' % (self.cfgdir, self.name))
		self.settings.load('%s' % 'db', {}, suppress=True)
		self.version = __version__

		# Load interface settings
		self.interface = self.setcfg('%s/%s' % (self.cfgdir, self.name))
		self.interface.load('%s-%s' % (self.name, interface), args, suppress=True)

		# Expose these for others to tap into
		self.process = process.Process

		# set logger path
		self.logdir = os.path.expanduser('~') + '%s/%s' % (self.cfgdir, self.name)

		initmsg = 'scrib v%s started' % self.version
		self.log(interface, self.name, initmsg)

		if self.settings.debug is 1:
			self.barf('DBG', initmsg)

	def barf(self, code, message, time=True, hour=True, log=False):
		barf.Barf(code, message, time, hour, log, logdir=self.cfgdir + '/')

	def setcfg(self, name):
		"""
		Simple wrapper for cfg.set()
		"""
		return cfg.set(name)

	def save(self):
		self.barf('SAV', 'Saving settings')
		self.settings.save()

	def shutdown(self, interface=False):
		"""
		Shuts us the scrib down.
		"""
		msg = 'Shutting down. Goodbye!'
		try:
			if 'quitmsg' in self.interface._defaults:
				msg = self.interface._defaults['quitmsg']
		except:
			pass

		try:
			if interface is not False:
				self.log(interface, self.name, msg)
				self.reply(interface, self.name, msg, reply='', bypass=True)
		except AttributeError as e:
			self.barf('ERR', 'scrib.shutdown: %s' % e)
			sys.exit()

		self.save()
		self.barf('MSG', msg)
		sys.exit()

	def log(self, interface, sender, message):
		timestamp = str(datetime.time(1, 2, 3))
		logme = "[%s] %s <%s> %s" % (timestamp, interface, sender, message)
		logger.Logger(self.logdir, '%s' % self.name).log(logme)

	def num_words(self, interface):
		return self.process(self.name, interface).wordcount()

	def reply(self, interface, name, message, reply='', learn=True, bypass=False):
		"""
		Send reply, or bypass if told to.
		"""
		if bypass is True:
			return message

		if message[1:] == '!':
			if self.settings.debug is 1:
				self.barf('DBG', 'scrib.reply: Abort; is command.')
			return

		if '://' in message:
			if self.settings.debug is 1:
				self.barf('DBG', 'scrib.reply: Abort; link detected.')
			return

		try:
			ret = self.process(self.name, interface).msg(name, message, reply, learn)
			return ret

		except UnicodeDecodeError as e:
			# Don't need to crash every goddamn time, yo.
			self.barf('ERR', 'scrib.reply: %s' % e)
			self.barf('ACT', 'Ignoring...')
			# Try again
			if self.settings.debug is 1:
				self.barf('DBG', 'Retrying last message...')
			self.reply(interface, name, message, reply, learn, bypass)
