#! /usr/bin/env python
from datetime import datetime
import random

from scoundrels import barf
from scrib import __appname__, cfg, clean, db

import redis


class Process:
	"""
	Setting up processing.
	"""
	def __init__(self, name, interface):
		self.name = name
		self.cfgdir = '/.%s/%s' % (__appname__, self.name)
		self.settings = cfg.set(self.cfgdir)
		# Don't set defaults here.
		self.settings.load('db', {}, suppress=True)
		self.interface = cfg.set(self.cfgdir)
		# Don't set defaults here.
		self.interface.load('%s-%s' % (self.name, interface), {}, suppress=True)
		self.barf = barf.Barf
		self.clean = clean.Clean(self.settings.debug)
		self.messages_to_generate = 1  # In case you want to allow multiple responses
		self.separator = ' '
		self.prefix = 'irc'

		self.redis_conn = redis.Redis(host='ataraxia.scndrls.net', port=6379, db=0)

	def msg(self, name, data, learn, reply=''):
		messages = []
		if self.settings.debug is 1:
			self.barf('DBG', 'process.msg: %s' % data)

		if data.startswith('/'):
			return

		if learn is True:
			self.learn(name, data)

		if reply:
			best_message = ''
			sentence = self.clean.split(self.clean.sanitize(data))
			for words in sentence:
				key = self.separator.join(words[:-1])

			for i in range(self.messages_to_generate):
				generated = self.generate(seed=key)
				if len(generated) > len(best_message):
						best_message = generated

			if best_message:
				messages.append(best_message)

		if len(messages):
			if self.settings.debug is 1:
				self.barf('DBG', 'process.msg: replying')
			return random.choice(messages)

		if self.settings.debug is 1:
				self.barf('DBG', 'process.msg: not replying')

	def split_sentences(self, message):
		message = str(message)
		try:
			try:
				message = [x for x in map(str.strip, message.split('.')) if x]
			except AttributeError as e:
				self.barf('ERR', 'process.split_sentences: AttributeError: %s' % e)
			
			return message

		except Exception as e:
			self.barf('ERR', 'process.split_sentences: %s' % e)

	def find_ngram(self, input, n=3):
		return zip(*[input[i:] for i in range(n)])

	def learn(self, name, message, key):
		wordlist = self.clean.split(self.clean.sanitize(message))
		sentence = message.strip()
		msg = ''

		if sentence is None:
			self.barf('MSG', "Sentence is empty; aborting.")
			return

		if self.settings.debug is 1:
			self.barf('DBG', 'Learning: `%s`' % sentence)
		try:
			s = db.Sentence()
			s.src = name
			s.src_input = sentence
			s.sentiment = self.sentiment(s.src_input)
			s.timestamp = datetime.utcnow()
			s.insert()
		except Exception as e:
			msg = '! Sentence learning failed: %s' % e.message
			self.barf('ERR', msg)

		try:
			for words in wordlist:
				key = self.separator.join(words[:-1])
				word = words[-1].strip()
				try:
					self.redis_conn.sadd(self.make_key(key), word)
					if self.settings.debug is 1:
						self.barf('DBG', '%s, %s added to redis' % (self.make_key(key), word))
				except Exception as e:
					self.barf('ERR', e.message)

				w = db.Word()
				w.key = key
				w.word = word
				w.insert()
				if self.settings.debug is 1:
					self.barf('DBG', '%s added to database.' % word)

			msg = 'Sentence and words learned.'
		except Exception as e:
			msg = '! Word learning failed: %s' % e.message
			self.barf('ERR', msg)

		return msg

	def wordcount(self):
		return db.Word().count()

	def sentiment(self, sentence):
		return 0  # this sets it to neutral, for now.

	def make_key(self, k):
		return '-'.join((self.prefix, k))

	def generate(self, seed):
		"""
		GENERATE ALL THE THINGS
		"""
		key = seed

		if self.settings.debug is 1:
			self.barf('DBG', 'process.generate: Seed: %s ' % seed)
		
		gen_words = []

		for i in xrange(self.interface.maxwords):
			words = key.split(self.separator)

			if self.settings.debug is 1:
				self.barf('DBG', 'process.generate gen_words: key: %s' % key)
				self.barf('DBG', 'process.generate gen_words: %s' % words[0])
			gen_words.append(words[0])
			# next_word = db.Word().get_random(key)
			next_word = self.redis_conn.srandmember(self.make_key(key))

			if self.settings.debug is 1:
				self.barf('DBG', 'process.generate: Next word: %s' % next_word)
			if not next_word:
				break

			key = self.separator.join(words[1:] + [next_word])

		return ' '.join(gen_words)
